"""generate openapi docs."""
import json
import argparse
from pkg_resources import get_distribution

from pydantic_openapi_helper.core import get_openapi
from pydantic_openapi_helper.inheritance import class_mapper
from ladybug_display_schema.geometry2d import Vector2D, Point2D, Ray2D, LineSegment2D, \
    Polyline2D, Arc2D, Polygon2D, Mesh2D
from ladybug_display_schema.geometry3d import Vector3D, Point3D, Ray3D, Plane, \
    LineSegment3D, Polyline3D, Arc3D, Face3D, Mesh3D, Polyface3D, Sphere, Cone, Cylinder
from ladybug_display_schema.display2d import DisplayVector2D, DisplayPoint2D, \
    DisplayRay2D, DisplayLineSegment2D, DisplayPolyline2D, DisplayArc2D, \
    DisplayPolygon2D, DisplayMesh2D
from ladybug_display_schema.display3d import DisplayVector3D, DisplayPoint3D, \
    DisplayRay3D, DisplayPlane, DisplayLineSegment3D, DisplayPolyline3D, DisplayArc3D, \
    DisplayFace3D, DisplayMesh3D, DisplayPolyface3D, DisplaySphere, DisplayCone, \
    DisplayCylinder
from ladybug_display_schema.visualization import VisualizationSet, VisualizationMetaData

parser = argparse.ArgumentParser(description='Generate OpenAPI JSON schemas')

parser.add_argument('--version', help='Set the version of the new OpenAPI Schema')

args = parser.parse_args()

if args.version:
    VERSION = args.version.replace('v', '')
else:
    VERSION = '.'.join(get_distribution('ladybug_display_schema').version.split('.')[:3])
lic_url = "https://github.com/ladybug-tools/ladybug-display-schema/blob/master/LICENSE"

info = {
    "description": "",
    "version": VERSION,
    "title": "",
    "contact": {
        "name": "Ladybug Tools",
        "email": "info@ladybug.tools",
        "url": "https://github.com/ladybug-tools/ladybug-display-core"
    },
    "x-logo": {
        "url": "https://www.ladybug.tools/assets/img/ladybug-large.png",
        "altText": "Ladybug logo"
    },
    "license": {
        "name": "MIT",
        "url": lic_url
    }
}

all_geo = [
    Vector2D, Point2D, Ray2D, LineSegment2D, Polyline2D, Arc2D, Polygon2D, Mesh2D,
    Vector3D, Point3D, Ray3D, Plane, LineSegment3D, Polyline3D, Arc3D, Face3D, Mesh3D,
    Polyface3D, Sphere, Cone, Cylinder
]

all_dis = [
    DisplayVector2D, DisplayPoint2D, DisplayRay2D, DisplayLineSegment2D,
    DisplayPolyline2D, DisplayArc2D, DisplayPolygon2D, DisplayMesh2D,
    DisplayVector3D, DisplayPoint3D, DisplayRay3D, DisplayPlane, DisplayLineSegment3D,
    DisplayPolyline3D, DisplayArc3D, DisplayFace3D, DisplayMesh3D,
    DisplayPolyface3D, DisplaySphere, DisplayCone, DisplayCylinder
]

modules = [
    {'module': [VisualizationSet, VisualizationMetaData], 'name': 'Visualization'},
    {'module': all_geo, 'name': 'Geometry'},
    {'module': all_dis, 'name': 'Display'}
]


def _process_name(name):
    """Process module name."""
    new_name = '-'.join(n.lower() for n in name.split())
    return new_name


for module in modules:
    # generate open api schema
    print(f'Generating {module["name"]} documentation...')

    external_docs = {
        "description": "OpenAPI Specification with Inheritance",
        "url": f"./{_process_name(module['name'])}_inheritance.json"
    }

    openapi = get_openapi(
        module['module'],
        title=f'Ladybug {module["name"]} Schema',
        description=f'Ladybug {_process_name(module["name"])} schema.',
        version=VERSION, info=info,
        external_docs=external_docs
    )

    # write out the base openAPI schema
    with open(f'./docs/{_process_name(module["name"])}.json', 'w') as out_file:
        json.dump(openapi, out_file, indent=2)

    # with inheritance
    openapi = get_openapi(
        module['module'],
        title=f'Ladybug {module["name"]} Schema',
        description=f'Documentation for Ladybug {_process_name(module["name"])} schema',
        version=VERSION, info=info,
        inheritance=True,
        external_docs=external_docs
    )

    # write out the OpenAPI with inheritance
    with open(f'./docs/{_process_name(module["name"])}_inheritance.json', 'w') \
            as out_file:
        json.dump(openapi, out_file, indent=2)

    # add the mapper file
    with open(f'./docs/{_process_name(module["name"])}_mapper.json', 'w') as out_file:
        json.dump(class_mapper(module['module']), out_file, indent=2)

# generate schema for mode with inheritance but without descriminator
# we will use this file for generating redocly - the full model is too big, and the
# model with inheritance and discriminators is renders incorrectly
external_docs = {
    "description": "OpenAPI Specification with Inheritance",
    "url": "./visualization_inheritance.json"
}

openapi = get_openapi(
    [VisualizationSet, VisualizationMetaData],
    title='Ladybug Visualization Schema',
    description='Documentation for Honeybee model schema',
    version=VERSION, info=info,
    inheritance=True,
    external_docs=external_docs,
    add_discriminator=False
)

with open('./docs/visualization_redoc.json', 'w') as out_file:
    json.dump(openapi, out_file, indent=2)



import os
from ladybug_display_schema.visualization import VisualizationSet

# target folder where all of the samples live
root = os.path.dirname(os.path.dirname(__file__))
target_folder = os.path.join(root, 'samples', 'vsf')


def test_visualization_set():
    file_path = os.path.join(target_folder, 'simple_vis.vsf')
    VisualizationSet.parse_file(file_path)


def test_visualization_set_detailed():
    file_path = os.path.join(target_folder, 'detailed_vis.vsf')
    VisualizationSet.parse_file(file_path)


def test_visualization_set_model():
    file_path = os.path.join(target_folder, 'model.vsf')
    VisualizationSet.parse_file(file_path)


def test_visualization_set_model_face_attr():
    file_path = os.path.join(target_folder, 'model_face_attr.vsf')
    VisualizationSet.parse_file(file_path)


def test_visualization_set_model_room_attr():
    file_path = os.path.join(target_folder, 'model_room_attr.vsf')
    VisualizationSet.parse_file(file_path)


def test_visualization_set_model_text_attr():
    file_path = os.path.join(target_folder, 'model_text_attr.vsf')
    VisualizationSet.parse_file(file_path)


def test_visualization_set_sunpath():
    file_path = os.path.join(target_folder, 'sunpath.vsf')
    VisualizationSet.parse_file(file_path)


def test_visualization_set_sunpath2d():
    file_path = os.path.join(target_folder, 'sunpath2d.vsf')
    VisualizationSet.parse_file(file_path)


def test_visualization_set_wind_profile():
    file_path = os.path.join(target_folder, 'wind_profile.vsf')
    VisualizationSet.parse_file(file_path)


def test_visualization_set_wind_rose():
    file_path = os.path.join(target_folder, 'wind_rose.vsf')
    VisualizationSet.parse_file(file_path)


def test_visualization_set_hourly_plot():
    file_path = os.path.join(target_folder, 'hourly_plot.vsf')
    VisualizationSet.parse_file(file_path)



"""Schemas for geometric display objects in 2D space."""
from typing import Union
from pydantic import Field, constr

from .base import SingleColorBase, LineCurveBase, DisplayModes, Default
from .geometry2d import Vector2D, Point2D, Ray2D, LineSegment2D, \
    Polyline2D, Arc2D, Polygon2D, Mesh2D


class DisplayVector2D(SingleColorBase):
    """A point object in 2D space with display properties."""

    type: constr(regex='^DisplayVector2D$') = 'DisplayVector2D'

    geometry: Vector2D = Field(
        ...,
        description='Vector2D for the geometry.'
    )


class DisplayPoint2D(SingleColorBase):
    """A point object in 2D space with display properties."""

    type: constr(regex='^DisplayPoint2D$') = 'DisplayPoint2D'

    geometry: Point2D = Field(
        ...,
        description='Point2D for the geometry.'
    )

    radius: Union[Default, float] = Field(
        Default(),
        ge=0,
        description='Number for the radius with which the point should be displayed '
        'in pixels (for the screen) or millimeters (in print).'
    )


class DisplayRay2D(SingleColorBase):
    """A ray object in 2D space with display properties."""

    type: constr(regex='^DisplayRay2D$') = 'DisplayRay2D'

    geometry: Ray2D = Field(
        ...,
        description='Ray2D for the geometry.'
    )


class DisplayLineSegment2D(LineCurveBase):
    """A single line segment face in 2D space with display properties."""

    type: constr(regex='^DisplayLineSegment2D$') = 'DisplayLineSegment2D'

    geometry: LineSegment2D = Field(
        ...,
        description='LineSegment2D for the geometry.'
    )


class DisplayPolyline2D(LineCurveBase):
    """A polyline in 2D space with display properties."""

    type: constr(regex='^DisplayPolyline2D$') = 'DisplayPolyline2D'

    geometry: Polyline2D = Field(
        ...,
        description='Polyline2D for the geometry.'
    )


class DisplayArc2D(LineCurveBase):
    """A single arc or circle in 2D space with display properties."""

    type: constr(regex='^DisplayArc2D$') = 'DisplayArc2D'

    geometry: Arc2D = Field(
        ...,
        description='Arc2D for the geometry.'
    )


class DisplayPolygon2D(LineCurveBase):
    """A single polygon in 2D space with display properties."""

    type: constr(regex='^DisplayPolygon2D$') = 'DisplayPolygon2D'

    geometry: Polygon2D = Field(
        ...,
        description='Polygon2D for the geometry.'
    )


class DisplayMesh2D(SingleColorBase):
    """A mesh in 2D space with display properties."""

    type: constr(regex='^DisplayMesh2D$') = 'DisplayMesh2D'

    geometry: Mesh2D = Field(
        ...,
        description='Mesh2D for the geometry.'
    )

    display_mode: DisplayModes = Field(
        DisplayModes.surface,
        description='Text to indicate the display mode (surface, wireframe, '
        'etc.). The DisplayModes enumeration contains all acceptable types.'
    )
